@@ -1,20 +1,9 @@
-// File_Mpeg_Descriptors - Info for MPEG files
-// Copyright (C) 2007-2011 MediaArea.net SARL, Info@MediaArea.net
-//
-// This library is free software: you can redistribute it and/or modify it
-// under the terms of the GNU Lesser General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
-// any later version.
-//
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-// GNU Lesser General Public License for more details.
-//
-// You should have received a copy of the GNU Lesser General Public License
-// along with this library. If not, see <http://www.gnu.org/licenses/>.
-//
-//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+/*  Copyright (c) MediaArea.net SARL. All Rights Reserved.
+ *
+ *  Use of this source code is governed by a BSD-style license that can
+ *  be found in the License.html file in the root of the source tree.
+ */
+
 //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 //
 // Information about MPEG files, Descriptors
@@ -27,10 +16,13 @@
 //---------------------------------------------------------------------------
 
 //---------------------------------------------------------------------------
-#include "MediaInfo/File__Analyze.h"
 #include "MediaInfo/Multiple/File_Mpeg4_Descriptors.h"
 #include "MediaInfo/Duplicate/File__Duplicate_MpegTs.h"
-#include <map>
+#if defined(MEDIAINFO_EIA608_YES) || defined(MEDIAINFO_EIA708_YES)
+    #include "MediaInfo/File__Analyze.h"
+#endif
+#include <cfloat>
+#include <set>
 //---------------------------------------------------------------------------
 
 namespace MediaInfoLib
@@ -51,16 +43,55 @@ struct complete_stream
     Ztring Duration_End;
     bool   Duration_End_IsUpdated;
     std::map<Ztring, Ztring> TimeZones; //Key is country code
-
+#if defined(MEDIAINFO_EIA608_YES) || defined(MEDIAINFO_EIA708_YES)
+    class service_desc_holder
+    {
+    public:
+        File__Analyze::servicedescriptors* ServiceDescriptors;
+        service_desc_holder() : ServiceDescriptors(NULL)
+        {
+        }
+        ~service_desc_holder()
+        {
+            reset();
+        }
+        void Clone_Desc(File__Analyze::servicedescriptors* src)
+        {
+            if (src)
+            {
+                reset(new File__Analyze::servicedescriptors);
+                *ServiceDescriptors = *src;
+            }
+            else
+            {
+                reset();
+            }
+        }
+        void reset(File__Analyze::servicedescriptors* new_ptr = NULL)
+        {
+            if(ServiceDescriptors)
+               delete ServiceDescriptors;
+            ServiceDescriptors = new_ptr;
+        }
+    };
+#endif
     //Per transport_stream
     struct transport_stream
+#if defined(MEDIAINFO_EIA608_YES) || defined(MEDIAINFO_EIA708_YES)
+        : public service_desc_holder
+#endif
     {
         bool HasChanged;
         std::map<std::string, Ztring> Infos;
-        struct program
+        struct program 
+#if defined(MEDIAINFO_EIA608_YES) || defined(MEDIAINFO_EIA708_YES)
+            : public service_desc_holder
+#endif
         {
             bool HasChanged;
             std::map<std::string, Ztring> Infos;
+            std::map<std::string, Ztring> ExtraInfos_Content;
+            std::map<std::string, Ztring> ExtraInfos_Option;
             std::map<Ztring, Ztring> EPGs;
             std::vector<int16u> elementary_PIDs;
             size_t StreamPos; //Stream_Menu
@@ -71,6 +102,7 @@ struct complete_stream
             bool   source_id_IsValid;
             bool   IsParsed;
             bool   IsRegistered;
+            bool   HasNotDisplayableStreams; //e.g. unknown stream, KLV, SCTE 35
             bool   Update_Needed_IsRegistered;
             bool   Update_Needed_StreamCount;
             bool   Update_Needed_StreamPos;
@@ -133,26 +165,87 @@ struct complete_stream
 
             //Constructor/Destructor
             program()
+            :
+                HasChanged(false),
+                StreamPos((size_t)-1),
+                registration_format_identifier(0x00000000),
+                pid(0x00000),
+                PCR_PID(0x0000),
+                source_id((int16u)-1),
+                source_id_IsValid(false),
+                IsParsed(false),
+                IsRegistered(false),
+                HasNotDisplayableStreams(false),
+                Update_Needed_IsRegistered(false),
+                Update_Needed_StreamCount(false),
+                Update_Needed_StreamPos(false),
+                Update_Needed_Info(false),
+                DVB_EPG_Blocks_IsUpdated(false),
+                Scte35(NULL)
+            {}
+
+            program(const program& p)
+            :
+                HasChanged(p.HasChanged),
+                Infos(p.Infos),
+                ExtraInfos_Content(p.ExtraInfos_Content),
+                ExtraInfos_Option(p.ExtraInfos_Option),
+                EPGs(p.EPGs),
+                elementary_PIDs(p.elementary_PIDs),
+                StreamPos(p.StreamPos),
+                registration_format_identifier(p.registration_format_identifier),
+                pid(p.pid),
+                PCR_PID(p.PCR_PID),
+                source_id(p.source_id),
+                source_id_IsValid(p.source_id_IsValid),
+                IsParsed(p.IsParsed),
+                IsRegistered(p.IsRegistered),
+                HasNotDisplayableStreams(p.HasNotDisplayableStreams),
+                Update_Needed_IsRegistered(p.Update_Needed_IsRegistered),
+                Update_Needed_StreamCount(p.Update_Needed_StreamCount),
+                Update_Needed_StreamPos(p.Update_Needed_StreamPos),
+                Update_Needed_Info(p.Update_Needed_Info),
+                DVB_EPG_Blocks_IsUpdated(p.DVB_EPG_Blocks_IsUpdated),
+                Scte35(p.Scte35)
             {
-                HasChanged=false;
-                StreamPos=(size_t)-1;
-                registration_format_identifier=0x00000000;
-                pid=0x00000;
-                PCR_PID=0x0000;
-                source_id=(int16u)-1;
-                source_id_IsValid=false;
-                IsParsed=false;
-                IsRegistered=false;
-                Update_Needed_IsRegistered=false;
-                Update_Needed_StreamCount=false;
-                Update_Needed_StreamPos=false;
-                Update_Needed_Info=false;
-                DVB_EPG_Blocks_IsUpdated=false;
-                Scte35=NULL;
+                #if defined(MEDIAINFO_EIA608_YES) || defined(MEDIAINFO_EIA708_YES)
+                Clone_Desc(p.ServiceDescriptors);
+                #endif
+            }
+
+            program& operator=(const program& p)
+            {
+                #if defined(MEDIAINFO_EIA608_YES) || defined(MEDIAINFO_EIA708_YES)
+                Clone_Desc(p.ServiceDescriptors);
+                #endif
+                HasChanged=p.HasChanged;
+                Infos=p.Infos;
+                ExtraInfos_Content=p.ExtraInfos_Content;
+                ExtraInfos_Option=p.ExtraInfos_Option;
+                EPGs=p.EPGs;
+                elementary_PIDs=p.elementary_PIDs;
+                StreamPos=p.StreamPos;
+                registration_format_identifier=p.registration_format_identifier;
+                pid=p.pid;
+                PCR_PID=p.PCR_PID;
+                source_id=p.source_id;
+                source_id_IsValid=p.source_id_IsValid;
+                IsParsed=p.IsParsed;
+                IsRegistered=p.IsRegistered;
+                HasNotDisplayableStreams=p.HasNotDisplayableStreams;
+                Update_Needed_IsRegistered=p.Update_Needed_IsRegistered;
+                Update_Needed_StreamCount=p.Update_Needed_StreamCount;
+                Update_Needed_StreamPos=p.Update_Needed_StreamPos;
+                Update_Needed_Info=p.Update_Needed_Info;
+                DVB_EPG_Blocks_IsUpdated=p.DVB_EPG_Blocks_IsUpdated;
+                Scte35=p.Scte35;
+
+                return *this;
             }
         };
         typedef std::map<int16u, program> programs; //Key is program_number
         programs Programs; //Key is program_number
+        std::vector<int16u> programs_List;
         size_t   Programs_NotParsedCount;
 
         //Per IOD
@@ -185,7 +278,7 @@ struct complete_stream
 
         //ATSC
         int16u source_id; //Global
-        int16u source_id_IsValid;
+        bool   source_id_IsValid;
 
         transport_stream()
         {
@@ -232,14 +325,12 @@ struct complete_stream
         typedef std::vector<table_id*>              table_ids;
         table_ids                                   Table_IDs; //Key is table_id
         std::map<std::string, Ztring>               Infos;
-        std::map<int8u, Ztring>                     Languages; //Key is caption_service_number or 128+line21_field
-        struct teletext
-        {
-            std::map<std::string, Ztring>           Infos;
-        };
-        std::map<int16u, teletext>                  Teletexts; //Key is teletext_magazine_number
+        std::map<std::string, Ztring>               Infos_Option;
+        #if defined(MEDIAINFO_TELETEXT_YES)
+            std::map<int16u, File__Analyze::teletext> Teletexts; //Key is teletext_magazine_number;
+        #endif
         #if MEDIAINFO_TRACE
-            Ztring Element_Info;
+            Ztring Element_Info1;
         #endif //MEDIAINFO_TRACE
         stream_t                                    StreamKind;
         stream_t                                    StreamKind_FromDescriptor;
@@ -255,12 +346,28 @@ struct complete_stream
             int64u                                  TimeStamp_End_Offset;
             int16u                                  PCR_PID; //If this pid has no PCR, decide which PCR should be used
             bool                                    TimeStamp_End_IsUpdated;
+            float64                                 TimeStamp_InstantaneousBitRate_Current_Min;
+            float64                                 TimeStamp_InstantaneousBitRate_Current_Raw;
+            float64                                 TimeStamp_InstantaneousBitRate_Current_Max;
+            int64u                                  TimeStamp_InstantaneousBitRate_BitRateMode_IsCbr;
+            int64u                                  TimeStamp_InstantaneousBitRate_BitRateMode_IsVbr;
+            #if MEDIAINFO_ADVANCED
+                float64                             TimeStamp_InstantaneousBitRate_Min_Raw;
+                float64                             TimeStamp_InstantaneousBitRate_Max_Raw;
+                int64u                              TimeStamp_Distance_Min;
+                int64u                              TimeStamp_Distance_Max;
+                int64u                              TimeStamp_Distance_Total;
+                int64u                              TimeStamp_Distance_Count;
+                int64u                              TimeStamp_HasProblems;
+                std::vector<int64u>                 TimeStamp_Intermediate;
+            #endif // MEDIAINFO_ADVANCED
         #endif //MEDIAINFO_MPEGTS_PCR_YES
         int32u                                      registration_format_identifier;
         int16u                                      FMC_ES_ID;
         int16u                                      table_type; //ATSC
         int8u                                       stream_type;
         int8u                                       descriptor_tag;
+        int8u                                       DtsNeural_config_id;
         bool                                        FMC_ES_ID_IsValid;
         bool                                        Searching;
         bool                                        Searching_Payload_Start;
@@ -278,11 +385,18 @@ struct complete_stream
         bool                                        IsRegistered;
         bool                                        IsUpdated_IsRegistered;
         bool                                        IsUpdated_Info;
-        size_t                                      IsScrambled;
+        bool                                        CA_system_ID_MustSkipSlices;
+        bool                                        EBP_Marker_Detected;
+        size_t                                      Scrambled_Count;
+        int16u                                      CA_system_ID;
         int16u                                      SubStream_pid;
-        #if MEDIAINFO_IBI
+        #if MEDIAINFO_IBIUSAGE
             int64u                                  Ibi_SynchronizationOffset_BeginOfFrame;
-        #endif //MEDIAINFO_IBI
+        #endif //MEDIAINFO_IBIUSAGE
+        #if defined(MEDIAINFO_EIA608_YES) || defined(MEDIAINFO_EIA708_YES)
+            File__Analyze::servicedescriptors ServiceDescriptors;
+            bool                              ServiceDescriptors_IsPresent;
+        #endif
 
         //Constructor/Destructor
         stream()
@@ -302,12 +416,27 @@ struct complete_stream
                 TimeStamp_End_Offset=(int64u)-1;
                 PCR_PID=0x0000;
                 TimeStamp_End_IsUpdated=false;
+                TimeStamp_InstantaneousBitRate_Current_Min=0;
+                TimeStamp_InstantaneousBitRate_Current_Raw=0;
+                TimeStamp_InstantaneousBitRate_Current_Max=0;
+                TimeStamp_InstantaneousBitRate_BitRateMode_IsCbr=0;
+                TimeStamp_InstantaneousBitRate_BitRateMode_IsVbr=0;
+                #if MEDIAINFO_ADVANCED
+                    TimeStamp_InstantaneousBitRate_Min_Raw=DBL_MAX;
+                    TimeStamp_InstantaneousBitRate_Max_Raw=0;
+                    TimeStamp_Distance_Min=(int64u)-1;
+                    TimeStamp_Distance_Max=0;
+                    TimeStamp_Distance_Total=0;
+                    TimeStamp_Distance_Count=0;
+                    TimeStamp_HasProblems=0;
+                #endif // MEDIAINFO_ADVANCED
             #endif //MEDIAINFO_MPEGTS_PCR_YES
             registration_format_identifier=0x00000000;
             FMC_ES_ID=0x0000;
             table_type=0x0000;
             stream_type=(int8u)-1;
             descriptor_tag=(int8u)-1;
+            DtsNeural_config_id=(int8u)-1;
             FMC_ES_ID_IsValid=false;
             Searching=false;
             Searching_Payload_Start=false;
@@ -325,11 +454,17 @@ struct complete_stream
             IsRegistered=false;
             IsUpdated_IsRegistered=false;
             IsUpdated_Info=false;
-            IsScrambled=false;
+            Scrambled_Count=0;
+            CA_system_ID_MustSkipSlices=false;
+            CA_system_ID=0x0000;
+            EBP_Marker_Detected=false;
             SubStream_pid=0x0000;
-            #if MEDIAINFO_IBI
+            #if MEDIAINFO_IBIUSAGE
                 Ibi_SynchronizationOffset_BeginOfFrame=(int64u)-1;
-            #endif //MEDIAINFO_IBI
+            #endif //MEDIAINFO_IBIUSAGE
+            #if defined(MEDIAINFO_EIA608_YES) || defined(MEDIAINFO_EIA708_YES)
+                ServiceDescriptors_IsPresent=false;
+            #endif
         }
 
         ~stream()
@@ -388,6 +523,13 @@ struct complete_stream
                     #endif //MEDIAINFO_MPEGTS_PESTIMESTAMP_YES
                     ;
         }
+        void init(const size_t ID)
+        {
+            Searching_Payload_Start_Set(true);
+            Kind = complete_stream::stream::psi;
+            Table_IDs.resize(0x100);
+            Table_IDs[ID] = new complete_stream::stream::table_id;
+        }
     };
     typedef std::vector<stream*> streams;
     streams Streams; //Key is pid
@@ -397,22 +539,40 @@ struct complete_stream
 
     //ATSC
     int8u GPS_UTC_offset;
-    struct source
+    struct source 
     {
         std::map<int16u, Ztring> texts;
         struct atsc_epg_block
         {
-            struct event
+            struct event 
+#if defined(MEDIAINFO_EIA608_YES) || defined(MEDIAINFO_EIA708_YES)
+                : public service_desc_holder
+#endif
             {
                 int32u  start_time;
                 Ztring  duration;
                 Ztring  title;
-                std::map<int8u, Ztring>  Languages; //Key is caption_service_number or 128+line21_field
                 std::map<int16u, Ztring> texts;
 
-                event()
+                event() : start_time((int32u)-1)
+                {}
+
+                event(const event& e)
+                :
+                    start_time(e.start_time)
                 {
-                    start_time=(int32u)-1;
+                    #if defined(MEDIAINFO_EIA608_YES) || defined(MEDIAINFO_EIA708_YES)
+                    Clone_Desc(e.ServiceDescriptors);
+                    #endif
+                }
+                event& operator=(const event& e)
+                {
+                    #if defined(MEDIAINFO_EIA608_YES) || defined(MEDIAINFO_EIA708_YES)
+                    Clone_Desc(e.ServiceDescriptors);
+                    #endif
+                    start_time=e.start_time;
+
+                    return *this;
                 }
             };
 
@@ -432,20 +592,21 @@ struct complete_stream
     sources Sources; //Key is source_id
     bool Sources_IsUpdated; //For EPG ATSC
     bool Programs_IsUpdated; //For EPG DVB
-    bool NoPatPmt;
 
     //File__Duplicate
+    #if MEDIAINFO_DUPLICATE
     bool                                                File__Duplicate_HasChanged_;
     size_t                                              Config_File_Duplicate_Get_AlwaysNeeded_Count;
     std::vector<File__Duplicate_MpegTs*>                Duplicates_Speed;
     std::vector<std::vector<File__Duplicate_MpegTs*> >  Duplicates_Speed_FromPID;
     std::map<const String, File__Duplicate_MpegTs*>     Duplicates;
-    bool File__Duplicate_Get_From_PID (int16u pid)
+    bool File__Duplicate_Get_From_PID (int16u pid) const
     {
         if (Duplicates_Speed_FromPID.empty())
             return false;
         return !Duplicates_Speed_FromPID[pid].empty();
     }
+    #endif //MEDIAINFO_DUPLICATE
 
     //SpeedUp information
     std::vector<std::vector<size_t> >   StreamPos_ToRemove;
@@ -465,8 +626,11 @@ struct complete_stream
         GPS_UTC_offset=0;
         Sources_IsUpdated=false;
         Programs_IsUpdated=false;
-        NoPatPmt=false;
         StreamPos_ToRemove.resize(Stream_Max);
+        #if MEDIAINFO_DUPLICATE
+        File__Duplicate_HasChanged_ = false;
+        Config_File_Duplicate_Get_AlwaysNeeded_Count = 0;
+        #endif //MEDIAINFO_DUPLICATE
     }
 
     ~complete_stream()
@@ -474,12 +638,14 @@ struct complete_stream
         for (size_t StreamID=0; StreamID<Streams.size(); StreamID++)
             delete Streams[StreamID]; //Streams[StreamID]=NULL;
 
+        #if MEDIAINFO_DUPLICATE
         std::map<const String, File__Duplicate_MpegTs*>::iterator Duplicates_Temp=Duplicates.begin();
         while (Duplicates_Temp!=Duplicates.end())
         {
             delete Duplicates_Temp->second; //Duplicates_Temp->second=NULL
-            Duplicates_Temp++;
+            ++Duplicates_Temp;
         }
+        #endif //MEDIAINFO_DUPLICATE
     }
 };
 
@@ -513,6 +679,9 @@ public :
     File_Mpeg_Descriptors();
 
 private :
+    //Buffer - File header
+    void FileHeader_Parse();
+
     //Buffer - Per element
     void Header_Parse();
     void Data_Parse();
@@ -568,6 +737,37 @@ private :
     void Descriptor_2D() {Skip_XX(Element_Size, "Data");};
     void Descriptor_2E() {Skip_XX(Element_Size, "Data");};
     void Descriptor_2F();
+    void Descriptor_30() {Skip_XX(Element_Size, "Data");};
+    void Descriptor_31() {Skip_XX(Element_Size, "Data");};
+    void Descriptor_32() {Skip_XX(Element_Size, "Data");};
+    void Descriptor_33() {Skip_XX(Element_Size, "Data");};
+    void Descriptor_34() {Skip_XX(Element_Size, "Data");};
+    void Descriptor_35() {Skip_XX(Element_Size, "Data");};
+    void Descriptor_36() {Skip_XX(Element_Size, "Data");};
+    void Descriptor_37() {Skip_XX(Element_Size, "Data");};
+    void Descriptor_38();
+    void Descriptor_3F();
+    void Descriptor_3F_00() {Skip_XX(Element_Size-Element_Offset, "Data");};
+    void Descriptor_3F_01() {Skip_XX(Element_Size-Element_Offset, "Data");};
+    void Descriptor_3F_02() {Skip_XX(Element_Size-Element_Offset, "Data");};
+    void Descriptor_3F_03();
+    void Descriptor_3F_04() {Skip_XX(Element_Size-Element_Offset, "Data");};
+    void Descriptor_3F_05() {Skip_XX(Element_Size-Element_Offset, "Data");};
+    void Descriptor_3F_06() {Skip_XX(Element_Size-Element_Offset, "Data");};
+    void Descriptor_3F_07() {Skip_XX(Element_Size-Element_Offset, "Data");};
+    void Descriptor_3F_08();
+    void Descriptor_3F_09() {Skip_XX(Element_Size-Element_Offset, "Data");};
+    void Descriptor_3F_0A() {Skip_XX(Element_Size-Element_Offset, "Data");};
+    void Descriptor_3F_0B() {Skip_XX(Element_Size-Element_Offset, "Data");};
+    void Descriptor_3F_0C() {Skip_XX(Element_Size-Element_Offset, "Data");};
+    void Descriptor_3F_0D() {Skip_XX(Element_Size-Element_Offset, "Data");};
+    void Descriptor_3F_0E() {Skip_XX(Element_Size-Element_Offset, "Data");};
+    void Descriptor_3F_0F() {Skip_XX(Element_Size-Element_Offset, "Data");};
+    void Descriptor_3F_10() {Skip_XX(Element_Size-Element_Offset, "Data");};
+    void Descriptor_3F_11() {Skip_XX(Element_Size-Element_Offset, "Data");};
+    void Descriptor_3F_12() {Skip_XX(Element_Size-Element_Offset, "Data");};
+    void Descriptor_3F_13() {Skip_XX(Element_Size-Element_Offset, "Data");};
+    void Descriptor_3F_14();
     void Descriptor_40();
     void Descriptor_41();
     void Descriptor_42() {Skip_XX(Element_Size, "Data");};
@@ -631,7 +831,11 @@ private :
     void Descriptor_7C();
     void Descriptor_7D() {Skip_XX(Element_Size, "Data");};
     void Descriptor_7E() {Skip_XX(Element_Size, "Data");};
-    void Descriptor_7F() {Skip_XX(Element_Size, "Data");};
+    void Descriptor_7F();
+    void Descriptor_7F_06();
+    void Descriptor_7F_0F();
+    void Descriptor_7F_15();
+    void Descriptor_7F_19();
     void Descriptor_80() {Skip_XX(Element_Size, "Data");};
     void Descriptor_81();
     void Descriptor_86();
@@ -644,6 +848,14 @@ private :
     void Descriptor_A9() {Skip_XX(Element_Size, "Data");};
     void Descriptor_AA();
     void Descriptor_AB() {Skip_XX(Element_Size, "Data");};
+    void Descriptor_B0();
+    void Descriptor_C1();
+    void Descriptor_C4() {Skip_XX(Element_Size, "Data");};
+    void Descriptor_C8();
+    void Descriptor_DE();
+    void Descriptor_E9();
+    void Descriptor_FC();
+    void Descriptor_FD();
 
     //SCTE 35
     void CUEI_00();
@@ -654,11 +866,9 @@ private :
     void ATSC_multiple_string_structure(Ztring &Value, const char* Info);
     void Get_DVB_Text(int64u Size, Ztring &Value, const char* Info);
     void Skip_DVB_Text(int64u Size, const char* Info) {Ztring Temp; Get_DVB_Text(Size, Temp, Info);};
-    Ztring Date_MJD(int16u Date);
-    Ztring Time_BCD(int32u Time);
-    Ztring TimeHHMM_BCD(int16u Time);
-    Ztring Frequency_DVB__BCD(int32u Frequency);
-    Ztring OrbitalPosition_DVB__BCD(int32u OrbitalPosition);
+    static Ztring TimeHHMM_BCD(int16u Time);
+    static Ztring Frequency_DVB__BCD(int32u Frequency);
+    static Ztring OrbitalPosition_DVB__BCD(int32u OrbitalPosition);
 };
 
 } //NameSpace
